/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Joan Segura <joan.segura@rcsb.org>
 * @author Yana Rose <yana.rose@rcsb.org>
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 */

import { Viewer, ViewerProps } from '@rcsb/rcsb-molstar/build/src/viewer';
import { TmDetColorThemeProvider } from '../tmdet-extension/tmdet-color-theme';
import { PluginBehavior } from 'molstar/lib/mol-plugin/behavior';
import { PluginSpec } from 'molstar/lib/mol-plugin/spec';
import { MembraneOrientationPreset, TMDETMembraneOrientation } from '../tmdet-extension/behavior';
import { SyncBehaviors } from 'molstar/lib/mol-plugin/behavior/static/state';
import { PluginStateObject } from 'molstar/lib/mol-plugin-state/objects';
import { MembraneOrientationProvider, TmDetDescriptorCache } from '../tmdet-extension/prop';
import { MembraneOrientationRepresentationProvider } from '../tmdet-extension/representation';
import { TmDetLabelProvider } from '../tmdet-extension/labeling';
import { registerTmDetSymmetry } from '../tmdet-extension/symmetry';


/** package version, filled in at bundle build time */
declare const __RCSB_MOLSTAR_VERSION__: string;
export const RCSB_MOLSTAR_VERSION = typeof __RCSB_MOLSTAR_VERSION__ != 'undefined' ? __RCSB_MOLSTAR_VERSION__ : 'none';

/** unix time stamp, to be filled in at bundle build time */
declare const __BUILD_TIMESTAMP__: number;
export const BUILD_TIMESTAMP = typeof __BUILD_TIMESTAMP__ != 'undefined' ? __BUILD_TIMESTAMP__ : 'none';
export const BUILD_DATE = new Date(BUILD_TIMESTAMP);

export class TmViewer extends Viewer {

    constructor(elementOrId: string | HTMLElement, props: Partial<ViewerProps> = {}) {
        console.log("TmViewer constructor");
        super(elementOrId, props);
        //this.plugin.representation.structure.themes.colorThemeRegistry.add(TmDetColorThemeProvider);
        this.initBehavior();
    }

    public async initBehavior() {
        let tree = this.plugin.state.behaviors.build();


        this.plugin.builders.structure.representation.registerPreset(MembraneOrientationPreset);
        this.plugin.representation.structure.themes.colorThemeRegistry.add(TmDetColorThemeProvider);
        this.plugin.managers.lociLabels.addProvider(TmDetLabelProvider);

        // const behavior = PluginSpec.Behavior(TMDETMembraneOrientation);
        // this.plugin.customStructureProperties.register(MembraneOrientationProvider, false);
        // this.plugin.representation.structure.registry.add(MembraneOrientationRepresentationProvider);


        // // // Init custom properties 1st
        // let cat = PluginBehavior.getCategoryId(behavior.transformer);
        // console.log('Behavior', behavior);
        // console.log('Transformer', behavior.transformer);
        // if (cat === 'custom-props') {
        //     tree.to(PluginBehavior.getCategoryId(behavior.transformer)).apply(behavior.transformer, behavior.defaultParams, { ref: behavior.transformer.id });
        // }
        // // this.plugin.state.events.object.created.subscribe(o => {
        // //     if (!PluginStateObject.isBehavior(o.obj)) return;
        // //     console.log({ o });
        // // });
        // //o.obj.data.register(o.ref);
        // //behavior.re
        // console.log('behavior', behavior);
        // console.log(PluginBehavior.Categories);
    
        // // await this.plugin.runTask(this.plugin.state.behaviors.updateTree(tree, { doNotUpdateCurrent: true, doNotLogTiming: true }));

        // tree = this.plugin.state.behaviors.build();
        // //const cat = PluginBehavior.getCategoryId(behavior.transformer);
        // if (cat !== 'custom-props') {
        //     tree.to(PluginBehavior.getCategoryId(behavior.transformer)).apply(behavior.transformer, behavior.defaultParams, { ref: behavior.transformer.id });
        // }

        await this.plugin.runTask(this.plugin.state.behaviors.updateTree(tree, { doNotUpdateCurrent: true, doNotLogTiming: true }));
    }

}

